"""changes

Revision ID: 3382cd293e50
Revises: 6b385de534b7
Create Date: 2022-08-28 00:00:06.142778

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "3382cd293e50"
down_revision = "6b385de534b7"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""made session_id col for session info

Revision ID: de275a6ec226
Revises: 04074343ab1f
Create Date: 2017-08-13 00:36:32.087692

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "de275a6ec226"
down_revision = "04074343ab1f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("session_info", sa.Column("session_id", sa.Integer(), nullable=False))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("session_info", "session_id")
    # ### end Alembic commands ###



"""Renamed incorrect umap_id correctly

Revision ID: d36bdd1852b5
Revises: a3f5d5b17c97
Create Date: 2018-09-18 23:17:13.309317

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "d36bdd1852b5"
down_revision = "a3f5d5b17c97"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "object_umap_features",
        sa.Column("object_umap_features_id", sa.Integer(), nullable=False),
    )
    op.drop_column("object_umap_features", "object_pca_features_id")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "object_umap_features",
        sa.Column(
            "object_pca_features_id", mysql.INTEGER(display_width=11), nullable=False
        ),
    )
    op.drop_column("object_umap_features", "object_umap_features_id")
    # ### end Alembic commands ###



# HOW TO:
# This script is supposed to be run from the command line with two
# arguments:
# The name of the series for which you wanna run this process.
# The folder to save the images and annotations files to.
#
# Example:
# python web/make_facenet_retraining_data.py "Seinfeld" "sample_data/seinfeld_facenet_1"

#####################################################
#           Imports                                 #
#####################################################

# Base Imports
import os
# import sys
import json
import logging
from random import shuffle

# Local Imports
from hoodat_utils import s3
from hoodat_utils import models as m

# Library Imports
import pickle
from PIL import Image
from matplotlib import pyplot
from sqlalchemy.orm import Session
from sqlalchemy import create_engine
from keras.models import load_model
from sklearn.svm import SVC
from sklearn.metrics import accuracy_score
from sklearn.preprocessing import Normalizer
from sklearn.preprocessing import LabelEncoder
from numpy import asarray, uint8, savez_compressed, expand_dims

#####################################################
#           Functions                               #
#####################################################


class retrain_facenet():
    def __init__(self, env, series_name, save_dir, sqlalchemy_database_uri):
        self.env = env
        self.save_dir = save_dir
        self.series_name = series_name
        self.sqlalchemy_database_uri = sqlalchemy_database_uri
        self.engine = create_engine(self.sqlalchemy_database_uri)

    def execute(self,
                train_split=1,
                predict=False,
                upload_to_s3=False,
                s3_save_dir=None):
        self.make_local_dir()
        logging.info("Downloading model")
        self.download_model()
        logging.info("Model downloaded")
        self.session = Session(self.engine)
        series_character_labels = self.query_series_character_labels()
        # logging.info(f"series_character_labels head: {series_character_labels[:3]}")
        if train_split != 1:
            shuffle(series_character_labels)
            split_index = round(train_split * len(series_character_labels))
            series_characters_train = series_character_labels[:split_index]
            series_characters_test = series_character_labels[split_index:]
        else:
            series_characters_train = series_character_labels
            series_characters_test = []
        # series_character_labels = self.query_series_character_labels()
        # logging.info(f"series_characters_train head: {series_characters_train[:3]}")
        character_list = self.unique_characters(series_character_labels)
        logging.info("character_list: ")
        logging.info(character_list)
        for character in character_list:
            character_folder_train = f"{self.save_dir}/train/{character}"
            character_folder_test = f"{self.save_dir}/test/{character}"
            if not os.path.isdir(character_folder_train):
                # logging.info(f"Creating {character_folder_train}")
                os.makedirs(character_folder_train)
            if not os.path.isdir(character_folder_test):
                # logging.info(f"Creating {character_folder_test}")
                os.makedirs(character_folder_test)
        files_train = self.make_file_names(series_characters_train)
        files_test = self.make_file_names(series_characters_test)
        file_list_train = self.make_file_list(series_characters_train,
                                              files_train, "train")
        file_list_test = self.make_file_list(series_characters_test,
                                             files_test, "test")
        self.download_images(file_list_train)
        self.download_images(file_list_test)
        logging.info("Making compressed datasets")
        self.make_compressed_datasets()
        logging.info("Made compressed datasets")
        logging.info("Loading model")
        self.load_facenet()
        logging.info("Loaded model")
        logging.info("Making embeddings")
        self.make_embeddings_dataset()
        logging.info("Made embeddings")
        logging.info("Fitting model")
        self.fit_model()
        logging.info("Model Fitted")
        if predict is True:
            logging.info("Scoring train and test sets")
            self.predict_model()
        if upload_to_s3 is True:
            logging.info("Saving model")
            self.save_retrained_model()
            logging.info("Saved model")
            self.upload_model_to_s3(s3_save_dir)
            logging.info("Uploaded model to S3")
            logging.info("Saving label_encoder")
            self.save_label_encoder()
            logging.info("Saved label_encoder")
            self.upload_label_encoder_to_s3(s3_save_dir)
            logging.info("Uploaded label_encoder to S3")

    def make_local_dir(self):
        if not os.path.exists(self.save_dir):
            os.makedirs(self.save_dir)

    def query_series_character_labels(self):
        query = self.session.query(
            m.Series.series_id, m.Video.video_id, m.Video.video_type,
            m.Video.save_name, m.Frame.frame_id, m.Frame.frame_number,
            m.Object.object_id, m.Character_Label.character_label_id,
            m.Object.object_x, m.Object.object_y, m.Object.object_w,
            m.Object.object_h, m.Object.object_cascade,
            m.Character.character_name).filter(
                m.Series.series_name == self.series_name).join(m.Episode).join(
                    m.Video).join(m.Frame).join(m.Object).join(
                        m.Character_Label).join(m.Character)
        series_character_labels = query.all()
        logging.info("Number of labels: {0}".format(
            str(len(series_character_labels))))
        return series_character_labels

    def unique_characters(self, series_character_labels):
        character_list = list(
            set([
                x.character_name.replace(" ", "_")
                for x in series_character_labels
            ]))
        return character_list

    def make_file_names(self, series_character_labels):
        result = [
            f"f{x[5]}c{x[12]}x{x[8]}y{x[9]}w{x[10]}h{x[11]}.jpg"
            for x in series_character_labels
        ]
        return result

    def make_file_download_from_list(self, series_character_labels,
                                     file_names):
        files = []
        for i, x in enumerate(series_character_labels):
            files.append(
                f"static/frames/{x[2].lower().replace(' ', '_')}/{x[3]}/haar/{file_names[i]}"
            )
        # logging.info(f"Head of files: {files[:3]}")
        return files

    def make_file_download_to_list(self,
                                   series_character_labels,
                                   file_names,
                                   train_or_test="train"):
        files = []
        for i, x in enumerate(series_character_labels):
            files.append(
                f"{self.save_dir}/{train_or_test}/{x[-1].replace(' ', '_')}/{file_names[i]}"
            )
        # files = [f"{self.save_dir}/{x[-1].replace(' ', '_')}/" for x in series_character_labels]
        # logging.info(f"Head of characters: {files[:3]}")
        return files

    def make_file_list(self,
                       series_character_labels,
                       file_names,
                       train_or_test="train"):
        from_list = self.make_file_download_from_list(series_character_labels,
                                                      file_names)
        to_list = self.make_file_download_to_list(series_character_labels,
                                                  file_names, train_or_test)
        file_list = list(zip(from_list, to_list))
        return file_list

    def download_images(self, file_list):
        logging.info("Creating s3 connection")
        s3_conn = s3.s3_data(access_key=self.env.AWS_ACCESS_KEY_ID,
                             secret_key=self.env.AWS_SECRET_ACCESS_KEY)
        logging.info("Starting file downloads")
        for file_from_and_to in file_list:
            s3_conn.download_file(bucket=self.env.BUCKET,
                                  s3_loc=file_from_and_to[0],
                                  save_loc=file_from_and_to[1])
        logging.info("Files downloaded")

    def download_model(self):
        logging.info("Creating s3 connection")
        s3_conn = s3.s3_data(access_key=self.env.AWS_ACCESS_KEY_ID,
                             secret_key=self.env.AWS_SECRET_ACCESS_KEY)
        s3_conn.download_file(
            bucket=self.env.BUCKET,
            s3_loc="static/data/dags/retrain_facenet/keras-facenet/model/facenet_keras.h5",
            save_loc=f"{self.save_dir}/facenet_keras.h5")

    def load_facenet(self):
        self.model = load_model(f"{self.save_dir}/facenet_keras.h5")

    def read_face(self, filename, required_size=(160, 160)):
        pixels = pyplot.imread(filename)
        image = Image.fromarray((pixels * 255).astype(uint8))
        image = image.resize(required_size)
        face_array = asarray(image)
        return face_array

    # load images and extract faces for all images in a directory
    def load_faces(self, directory):
        faces = list()
        # enumerate files
        for filename in os.listdir(directory):
            # path
            path = directory + filename
            # get face
            face = self.read_face(path)
            # store
            faces.append(face)
        return faces

    # load a dataset that contains one subdir for each class that in turn contains images
    def load_dataset(self, directory):
        X, y = list(), list()
        # enumerate folders, on per class
        for subdir in os.listdir(directory):
            # path
            path = directory + subdir + '/'
            # skip any files that might be in the dir
            if not os.path.isdir(path):
                continue
            # load all faces in the subdirectory
            faces = self.load_faces(path)
            # create labels
            labels = [subdir for _ in range(len(faces))]
            # summarize progress
            logging.info('>loaded %d examples for class: %s' %
                         (len(faces), subdir))
            # store
            X.extend(faces)
            y.extend(labels)
        return asarray(X), asarray(y)

    def make_compressed_datasets(self):
        # load train dataset
        self.trainX, self.trainy = self.load_dataset(f'{self.save_dir}/train/')
        logging.info(self.trainX.shape, self.trainy.shape)
        # load test dataset
        self.testX, self.testy = self.load_dataset(f'{self.save_dir}/test/')
        # save arrays to one file in compressed format
        savez_compressed(f'{self.save_dir}/dataset.npz', self.trainX,
                         self.trainy, self.testX, self.testy)

    def get_embedding(self, model, face_pixels):
        # scale pixel values
        face_pixels = face_pixels.astype('float32')
        # standardize pixel values across channels (global)
        mean, std = face_pixels.mean(), face_pixels.std()
        face_pixels = (face_pixels - mean) / std
        # transform face into one sample
        samples = expand_dims(face_pixels, axis=0)
        # make prediction to get embedding
        yhat = model.predict(samples)
        return yhat[0]

    def make_embeddings_dataset(self):
        # Convert each face in the train set to an embedding
        self.newTrainX = list()
        for face_pixels in self.trainX:
            embedding = self.get_embedding(self.model, face_pixels)
            self.newTrainX.append(embedding)
        self.newTrainX = asarray(self.newTrainX)
        logging.info(self.newTrainX.shape)
        # Convert each face in the test set to an embedding
        self.newTestX = list()
        for face_pixels in self.testX:
            embedding = self.get_embedding(self.model, face_pixels)
            self.newTestX.append(embedding)
        self.newTestX = asarray(self.newTestX)
        logging.info(self.newTestX.shape)
        # Save arrays to one file in compressed format
        savez_compressed(f'{self.save_dir}/embeddings.npz', self.newTrainX,
                         self.trainy, self.newTestX, self.testy)

    def fit_model(self):
        # Normalize input vectors
        in_encoder = Normalizer(norm='l2')
        self.transformedTrainX = in_encoder.transform(self.newTrainX)
        self.transformedTestX = in_encoder.transform(self.newTestX)
        # Label encode targets
        self.label_encoder = LabelEncoder()
        self.label_encoder.fit(self.trainy)
        self.transformedTrainy = self.label_encoder.transform(self.trainy)
        self.transformedTesty = self.label_encoder.transform(self.testy)
        # Fit model
        self.svc_model = SVC(kernel='linear', probability=True)
        self.svc_model.fit(self.transformedTrainX, self.transformedTrainy)

    def predict_model(self):
        # Predict
        yhat_train = self.svc_model.predict(self.transformedTrainX)
        yhat_test = self.svc_model.predict(self.transformedTestX)
        # Score
        score_train = accuracy_score(self.transformedTrainy, yhat_train)
        score_test = accuracy_score(self.transformedTesty, yhat_test)
        # Summarize
        logging.info('Accuracy: train=%.3f, test=%.3f' %
                     (score_train * 100, score_test * 100))

    def save_retrained_model(self):
        model_file = f"{self.save_dir}/facenet_svc_model.sav"
        pickle.dump(self.svc_model, open(model_file, "wb"))

    def save_label_encoder(self):
        file_path = f"{self.save_dir}/label_encoding.json"
        self.label_mapping = dict(
            zip([
                str(x) for x in self.label_encoder.transform(
                    self.label_encoder.classes_)
            ], self.label_encoder.classes_))
        with open(file_path, "w", encoding="utf-8") as f:
            json.dump(self.label_mapping, f, ensure_ascii=False, indent=4)

    def upload_model_to_s3(self, s3_save_dir):
        logging.info("Creating s3 connection")
        s3_conn = s3.s3_data(access_key=self.env.AWS_ACCESS_KEY_ID,
                             secret_key=self.env.AWS_SECRET_ACCESS_KEY)
        s3_conn.upload_file(bucket=self.env.BUCKET,
                            s3_loc=f"{s3_save_dir}/facenet_svc_model.sav",
                            local_loc=f"{self.save_dir}/facenet_svc_model.sav")
        logging.info("Model uploaded to s3")

    def upload_label_encoder_to_s3(self, s3_save_dir):
        logging.info("Creating s3 connection")
        s3_conn = s3.s3_data(access_key=self.env.AWS_ACCESS_KEY_ID,
                             secret_key=self.env.AWS_SECRET_ACCESS_KEY)
        s3_conn.upload_file(bucket=self.env.BUCKET,
                            s3_loc=f"{s3_save_dir}/label_encoding.json",
                            local_loc=f"{self.save_dir}/label_encoding.json")
        logging.info("Label encoder uploaded to s3")


# if __name__ == "__main__":
#     SERIES_NAME = sys.argv[1]
#     SAVE_DIR = sys.argv[2]
#     SQLALCHEMY_DATABASE_URI = "mysql://{0}:{1}@{2}/{3}".format(
#         env.DB_USER_RDS, env.DB_PASS_RDS, env.DB_SERVICE_RDS, env.DB_NAME_RDS
#     )
#     fnd = facenet_data(
#         series_name=SERIES_NAME,
#         save_dir=SAVE_DIR,
#         sqlalchemy_database_uri=SQLALCHEMY_DATABASE_URI)
#     fnd.execute()
#     logging.info("Script complete")



"""session_info timestamp as integer

Revision ID: 04074343ab1f
Revises: f0c1aa4b2657
Create Date: 2017-08-13 00:09:28.767522

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "04074343ab1f"
down_revision = "f0c1aa4b2657"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""session_info auto increment

Revision ID: b587da5794d6
Revises: de275a6ec226
Create Date: 2017-08-13 00:42:09.284574

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "b587da5794d6"
down_revision = "de275a6ec226"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""Added not nullable to user_id

Revision ID: 554178515c27
Revises: 335e48271836
Create Date: 2017-07-30 18:09:29.519639

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "554178515c27"
down_revision = "335e48271836"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "videos",
        "user_id",
        existing_type=mysql.INTEGER(display_width=11),
        nullable=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "videos",
        "user_id",
        existing_type=mysql.INTEGER(display_width=11),
        nullable=True,
    )
    # ### end Alembic commands ###



"""added actor_groups

Revision ID: 8f5ca94205dc
Revises: 02d94cd8a1ce
Create Date: 2018-07-18 18:44:12.886902

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "8f5ca94205dc"
down_revision = "02d94cd8a1ce"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "actor_groups",
        sa.Column("actor_group_id", sa.Integer(), nullable=False),
        sa.Column("actor_group_name", sa.String(length=64), nullable=False),
        sa.Column("timestamp_add_actor_group", sa.String(length=10), nullable=False),
        sa.PrimaryKeyConstraint("actor_group_id"),
    )
    op.create_table(
        "actor_group_lookup",
        sa.Column("actor_group_lookup_id", sa.Integer(), nullable=False),
        sa.Column("actor_group_id", sa.Integer(), nullable=False),
        sa.Column("actor_id", sa.Integer(), nullable=False),
        sa.Column(
            "timestamp_add_actor_group_lookup", sa.String(length=10), nullable=False
        ),
        sa.ForeignKeyConstraint(
            ["actor_group_id"],
            ["actor_groups.actor_group_id"],
        ),
        sa.ForeignKeyConstraint(
            ["actor_id"],
            ["actors.actor_id"],
        ),
        sa.PrimaryKeyConstraint("actor_group_lookup_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("actor_group_lookup")
    op.drop_table("actor_groups")
    # ### end Alembic commands ###



"""changed pca_object_features id colname and added timestamp added col

Revision ID: 02d94cd8a1ce
Revises: 4fdaad670a35
Create Date: 2018-07-05 20:33:12.997287

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "02d94cd8a1ce"
down_revision = "4fdaad670a35"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "object_pca_features",
        sa.Column("object_pca_features_id", sa.Integer(), nullable=False),
    )
    op.add_column(
        "object_pca_features",
        sa.Column("timestamp_added", sa.String(length=10), nullable=False),
    )
    op.drop_column("object_pca_features", "pca_features_id")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "object_pca_features",
        sa.Column("pca_features_id", mysql.INTEGER(display_width=11), nullable=False),
    )
    op.drop_column("object_pca_features", "timestamp_added")
    op.drop_column("object_pca_features", "object_pca_features_id")
    # ### end Alembic commands ###



"""Relationship between episodes and videos

Revision ID: 6c12ddacf663
Revises: 9133ec0c2db6
Create Date: 2017-08-10 19:30:32.504111

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "6c12ddacf663"
down_revision = "9133ec0c2db6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("videos", sa.Column("episode_id", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "videos", "episodes", ["episode_id"], ["episode_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "videos", type_="foreignkey")
    op.drop_column("videos", "episode_id")
    # ### end Alembic commands ###



"""Added youtube_link column

Revision ID: 55b537e0c4db
Revises: d51287a30494
Create Date: 2017-07-30 11:22:34.921037

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "55b537e0c4db"
down_revision = "d51287a30494"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "videos", sa.Column("youtube_link", sa.String(length=128), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("videos", "youtube_link")
    # ### end Alembic commands ###



"""Changed videos.upload_timestamp to videos.timestamp_add_video

Revision ID: 335e48271836
Revises: a25b2b88671d
Create Date: 2017-07-30 17:53:34.021758

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "335e48271836"
down_revision = "a25b2b88671d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "videos", sa.Column("timestamp_add_video", sa.String(length=10), nullable=False)
    )
    op.drop_column("videos", "upload_timestamp")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "videos",
        sa.Column("upload_timestamp", mysql.VARCHAR(length=10), nullable=False),
    )
    op.drop_column("videos", "timestamp_add_video")
    # ### end Alembic commands ###



"""add haar_cascade table

Revision ID: c4d469256008
Revises: 08a49ff86510
Create Date: 2018-03-20 19:10:40.348304

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c4d469256008"
down_revision = "08a49ff86510"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "haar_cascades",
        sa.Column("haar_cascade_id", sa.Integer(), nullable=False),
        sa.Column("haar_cascade_name", sa.String(length=32), nullable=False),
        sa.Column("file", sa.String(length=48), nullable=False),
        sa.PrimaryKeyConstraint("haar_cascade_id", "haar_cascade_name", "file"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("haar_cascades")
    # ### end Alembic commands ###



"""changes

Revision ID: c103233a6a69
Revises: 3382cd293e50
Create Date: 2022-08-28 00:02:03.942691

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c103233a6a69"
down_revision = "3382cd293e50"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""Added faces table to database

Revision ID: aa0ccca01f54
Revises: d36bdd1852b5
Create Date: 2020-05-15 17:20:56.645157

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "aa0ccca01f54"
down_revision = "d36bdd1852b5"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "faces",
        sa.Column("face_id", sa.Integer(), nullable=False),
        sa.Column("object_id", sa.Integer(), nullable=True),
        sa.Column("video_id", sa.Integer(), nullable=True),
        sa.Column("frame_id", sa.Integer(), nullable=True),
        sa.Column("episode_id", sa.Integer(), nullable=True),
        sa.Column("object_number", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["episode_id"],
            ["episodes.episode_id"],
        ),
        sa.ForeignKeyConstraint(
            ["frame_id"],
            ["frames.frame_id"],
        ),
        sa.ForeignKeyConstraint(
            ["object_id"],
            ["objects.object_id"],
        ),
        sa.ForeignKeyConstraint(
            ["video_id"],
            ["videos.video_id"],
        ),
        sa.PrimaryKeyConstraint("face_id"),
        sa.UniqueConstraint("video_id", "frame_id", "object_number", name="object"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("faces")
    # ### end Alembic commands ###



"""test revision with autogenerate

Revision ID: bc0978602558
Revises: 63fcb6f73f50
Create Date: 2017-07-29 12:30:02.705362

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "bc0978602558"
down_revision = "63fcb6f73f50"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("migrate_version")
    op.create_foreign_key(
        None, "face_classifications", "frames", ["frame_id"], ["frame_id"]
    )
    op.create_foreign_key(
        None, "face_classifications", "faces", ["face_id"], ["face_id"]
    )
    op.create_foreign_key(
        None, "face_classifications", "videos", ["video_id"], ["video_id"]
    )
    op.create_foreign_key(None, "face_classifications", "users", ["user_id"], ["id"])
    op.create_foreign_key(None, "faces", "videos", ["video_id"], ["video_id"])
    op.create_foreign_key(None, "faces", "users", ["user_id"], ["id"])
    op.create_foreign_key(None, "faces", "frames", ["frame_id"], ["frame_id"])
    op.create_foreign_key(None, "frames", "videos", ["video_id"], ["video_id"])
    op.create_foreign_key(None, "frames", "users", ["user_id"], ["id"])
    op.create_unique_constraint(None, "videos", ["video_display_name"])
    op.create_unique_constraint(None, "videos", ["video_name"])
    op.create_foreign_key(None, "videos", "users", ["user_id"], ["id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "videos", type_="foreignkey")
    op.drop_constraint(None, "videos", type_="unique")
    op.drop_constraint(None, "videos", type_="unique")
    op.drop_constraint(None, "frames", type_="foreignkey")
    op.drop_constraint(None, "frames", type_="foreignkey")
    op.drop_constraint(None, "faces", type_="foreignkey")
    op.drop_constraint(None, "faces", type_="foreignkey")
    op.drop_constraint(None, "faces", type_="foreignkey")
    op.drop_constraint(None, "face_classifications", type_="foreignkey")
    op.drop_constraint(None, "face_classifications", type_="foreignkey")
    op.drop_constraint(None, "face_classifications", type_="foreignkey")
    op.drop_constraint(None, "face_classifications", type_="foreignkey")
    op.create_table(
        "migrate_version",
        sa.Column("repository_id", mysql.VARCHAR(length=250), nullable=False),
        sa.Column("repository_path", mysql.TEXT(), nullable=True),
        sa.Column(
            "version",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.PrimaryKeyConstraint("repository_id"),
        mysql_default_charset="utf8",
        mysql_engine="InnoDB",
    )
    # ### end Alembic commands ###



"""session_info model

Revision ID: f0c1aa4b2657
Revises: 8ff7642a7c59
Create Date: 2017-08-13 00:07:54.003071

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "f0c1aa4b2657"
down_revision = "8ff7642a7c59"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "session_info",
        sa.Column("ip_timestamp", sa.String(length=50), nullable=False),
        sa.Column("ip", sa.String(length=16), nullable=False),
        sa.Column("timestamp", sa.Numeric(), nullable=False),
        sa.Column("route_rule", sa.String(length=100), nullable=False),
        sa.PrimaryKeyConstraint("ip_timestamp"),
    )
    op.create_unique_constraint("episode_video", "videos", ["video_id", "episode_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("episode_video", "videos", type_="unique")
    op.drop_table("session_info")
    # ### end Alembic commands ###



"""add columns to actors

Revision ID: 1b019da65d4f
Revises: 8fd3900c2c9e
Create Date: 2017-07-29 17:03:30.977180

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "1b019da65d4f"
down_revision = "8fd3900c2c9e"
branch_labels = None
depends_on = None


def upgrade():
    pass


def downgrade():
    pass



"""
A script that holds the definitions of the views in hoodat
"""

view_queries = {}

view_queries[
    "single_view_labels"
] = """
SELECT
    cl.character_label_id,
    cl.character_id,
    cl.timestamp_labelled,
    cl.object_id,
    c.character_name,
    c.actor_id,
    a.actor_name,
    c.series_id,
    s.series_name,
    o.object_number,
    o.object_cascade,
    o.object_x,
    o.object_y,
    o.object_w,
    o.object_h,
    o.video_id,
    v.video_name,
    v.video_type,
    o.episode_id,
    o.frame_id,
    f.frame_number,
    e.episode_number,
    s2.season_id,
    s2.season_number
FROM character_labels cl
LEFT JOIN `characters` c
ON cl.character_id = c.character_id
LEFT JOIN `actors` a
ON c.actor_id = a.actor_id
LEFT JOIN series s
ON c.series_id = s.series_id
LEFT JOIN objects o
ON cl.object_id = o.object_id
LEFT JOIN videos v
ON o.video_id = v.video_id
LEFT JOIN frames f
ON o.frame_id = f.frame_id
LEFT JOIN episodes e
ON o.episode_id = e.episode_id
LEFT JOIN seasons s2
ON e.season_id = s2.season_id
"""

view_queries[
    "single_view_frames"
] = """
SELECT
    f.frame_id,
    f.frame_number,
    s.series_name,
    f.video_id,
    v.video_name,
    v.video_type,
    e.episode_id,
    e.episode_number,
    s2.season_id,
    s2.season_number
FROM frames f
LEFT JOIN videos v
ON f.video_id = v.video_id
LEFT JOIN episodes e
ON v.episode_id = e.episode_id
LEFT JOIN series s
ON v.series_id = s.series_id
LEFT JOIN seasons s2
ON e.season_id = s2.season_id
"""


if __name__ == "__main__":
    import os
    from hoodat_utils.hoodat_sqlalchemy import HoodatSqlalchemy

    uri = os.environ.get("HOODAT_SQLALCHEMY_URI")
    hsa = HoodatSqlalchemy(sqlalchemy_database_uri=uri)
    create_view_result = hsa.create_views(views_obj=view_queries, overwrite=True)
    print(create_view_result)



#####################################################
#           Imports                                 #
#####################################################

# # Base Imports
import os
import json
import logging
from time import time
# import sys
# from random import shuffle

# # Local Imports
from hoodat_utils import s3
from hoodat_utils import models as m
from hoodat_utils.hoodat_sqlalchemy import HoodatSqlalchemy

# # Library Imports
import pickle
import pandas as pd
from PIL import Image
from matplotlib import pyplot
from keras.models import load_model
# from sqlalchemy.orm import Session
from sqlalchemy import create_engine
from numpy import asarray, uint8, savez_compressed, expand_dims
# from sklearn.svm import SVC
# from sklearn.metrics import accuracy_score
from sklearn.preprocessing import Normalizer
# from sklearn.preprocessing import LabelEncoder
# from numpy import asarray, load, uint8, savez_compressed, expand_dims

#####################################################
#           Functions                               #
#####################################################


class facenet_embeddings():
    def __init__(self,
                 env,
                 series_name,
                 run_name,
                 video,
                 folder,
                 save_dir,
                 sqlalchemy_database_uri,
                 use_complete_tables=False):
        self.env = env
        self.series_name = series_name
        self.run_name = run_name
        self.video = video
        self.folder = folder
        self.sqlalchemy_database_uri = sqlalchemy_database_uri
        self.save_dir = save_dir
        self.engine = create_engine(self.sqlalchemy_database_uri)
        self.hoodat_sql = HoodatSqlalchemy(self.sqlalchemy_database_uri)
        self.use_complete_tables = use_complete_tables
        if self.use_complete_tables:
            self.s3_haar_key = f"static/frames/{self.folder}/{self.video}/haar_complete/"
        else:
            self.s3_haar_key = f"static/frames/{self.folder}/{self.video}/haar/"

    def make_embeddings(self):
        # 0. Make local save directory
        self.make_local_dir()
        # 1. Create faces query for sql
        # video_faces = self.query_video_faces()
        # 2. Download faces from S3
        self.download_images()
        self.X = asarray(self.load_faces(f"{self.save_dir}/images/"))
        # 3. Download model from S3
        self.download_file(
            s3_loc="static/data/dags/retrain_facenet/keras-facenet/model/facenet_keras.h5",
            save_loc=f"{self.save_dir}/facenet_keras.h5")
        self.load_facenet()
        # 4. Make Embeddings
        self.make_embeddings_dataset()
        # 5. Save to S3
        self.upload_embeddings_to_s3(
            s3_save_dir=f"static/data/dags/haar_facenet_embeddings_complete/video={self.video}/model={self.run_name}"
        )

    def make_local_dir(self):
        if not os.path.exists(self.save_dir):
            os.makedirs(self.save_dir)

    # def query_video_faces(self):
    #     query = self.session.query(
    #         # m.Series.series_id,
    #         m.Video.video_id, m.Video.video_type, m.Video.save_name,
    #         m.Frame.frame_id, m.Frame.frame_number, m.Object.object_id, m.Object.object_x,
    #         m.Object.object_y, m.Object.object_w, m.Object.object_h,
    #         m.Object.object_cascade
    #     ).filter(
    #         m.Video.save_name == self.video
    #     ).join(m.Frame).join(m.Object)
    #     result = query.all()
    #     logging.info("Number of records: {0}".format(
    #         str(len(result))))
    #     return result

    def download_images(self):
        logging.info("Creating s3 connection")
        s3_conn = s3.s3_data(access_key=self.env.AWS_ACCESS_KEY_ID,
                             secret_key=self.env.AWS_SECRET_ACCESS_KEY)
        logging.info("Starting file downloads")
        s3_conn.download_dir(region=self.env.AWS_DEFAULT_REGION,
                             bucket=self.env.BUCKET,
                             s3_loc=self.s3_haar_key,
                             save_loc=f"{self.save_dir}/images/")
        logging.info("Files downloaded")

    def download_file(self, s3_loc, save_loc):
        logging.info("Creating s3 connection")
        s3_conn = s3.s3_data(access_key=self.env.AWS_ACCESS_KEY_ID,
                             secret_key=self.env.AWS_SECRET_ACCESS_KEY)
        s3_conn.download_file(bucket=self.env.BUCKET,
                              s3_loc=s3_loc,
                              save_loc=save_loc)

    def load_facenet(self):
        logging.info("Loading facenet model")
        self.model = load_model(f"{self.save_dir}/facenet_keras.h5")
        logging.info("Facenet model loaded")

    def read_face(self, filename, required_size=(160, 160)):
        pixels = pyplot.imread(filename)
        image = Image.fromarray((pixels * 255).astype(uint8))
        image = image.resize(required_size)
        face_array = asarray(image)
        return face_array

    # load images and extract faces for all images in a directory
    def load_faces(self, directory):
        faces = list()
        self.faces_files = os.listdir(directory)
        # enumerate files
        for filename in self.faces_files:
            # path
            path = directory + filename
            # get face
            face = self.read_face(path)
            # store
            faces.append(face)
        return faces

    def get_embedding(self, model, face_pixels):
        # scale pixel values
        face_pixels = face_pixels.astype('float32')
        # standardize pixel values across channels (global)
        mean, std = face_pixels.mean(), face_pixels.std()
        face_pixels = (face_pixels - mean) / std
        # transform face into one sample
        samples = expand_dims(face_pixels, axis=0)
        # make prediction to get embedding
        yhat = model.predict(samples)
        return yhat[0]

    def make_embeddings_dataset(self):
        logging.info("Making embeddings dataset")
        # Convert each face in the data set to an embedding
        self.newX = list()
        for face_pixels in self.X:
            embedding = self.get_embedding(self.model, face_pixels)
            self.newX.append(embedding)
        self.newX = asarray(self.newX)
        logging.info(self.X.shape)
        self.X = self.newX
        logging.info("Saving embeddings")
        # Save arrays to one file in compressed format
        savez_compressed(f'{self.save_dir}/embeddings.npz', self.X)
        logging.info("Embeddings saved")

    def upload_embeddings_to_s3(self, s3_save_dir):
        logging.info("Creating s3 connection")
        s3_conn = s3.s3_data(access_key=self.env.AWS_ACCESS_KEY_ID,
                             secret_key=self.env.AWS_SECRET_ACCESS_KEY)
        logging.info(f"Uploading embeddings file to {s3_save_dir}")
        s3_conn.upload_file(bucket=self.env.BUCKET,
                            s3_loc=f"{s3_save_dir}/embeddings.npz",
                            local_loc=f"{self.save_dir}/embeddings.npz")
        logging.info("Embeddings uploaded to s3")

    def predict_embeddings(self):
        # if video is not None:
        #     self.video = video
        # if model is not None:
        #     self.model = model
        # if self.X is None:
        #     self.X = self.download_file(
        #         s3_loc=f"static/data/dags/haar_facenet_embeddings/video={self.video}/model={self.run_name}/embeddings.npz",
        #         save_loc=f"{self.save_dir}/embeddings.npz")
        # svc_s3_loc = f"static/data/dags/retrain_facenet/retrained_models/{self.run_name}/facenet_svc_model.sav"
        # svc_save_loc = f"{self.save_dir}/facenet_svc_model.sav"
        # logging.info(f"Downloading {svc_s3_loc}")
        # self.download_file(
        #     s3_loc=svc_s3_loc,
        #     save_loc=svc_save_loc)
        # self.svc_model = pickle.load(open(svc_save_loc, "rb"))
        self.load_svc_model()
        # def predict_model(self):
        # Label encode targets
        # out_encoder = LabelEncoder()
        # out_encoder.fit(self.trainy)
        # self.transformedTrainy = out_encoder.transform(self.trainy)
        # Normalize input vectors
        in_encoder = Normalizer(norm='l2')
        self.transformedX = in_encoder.transform(self.X)
        # Predict
        yhat_X = self.svc_model.predict(self.transformedX)
        # Read encoder
        self.label_encoder = self.load_label_encoder()
        # self.faces_files
        return yhat_X

    def load_svc_model(self):
        svc_s3_loc = f"static/data/dags/retrain_facenet/retrained_models/{self.run_name}/facenet_svc_model.sav"
        svc_save_loc = f"{self.save_dir}/facenet_svc_model.sav"
        logging.info(f"Downloading {svc_s3_loc}")
        self.download_file(s3_loc=svc_s3_loc, save_loc=svc_save_loc)
        self.svc_model = pickle.load(open(svc_save_loc, "rb"))

    def load_label_encoder(self):
        label_encoder_s3_loc = f"static/data/dags/retrain_facenet/retrained_models/{self.run_name}/label_encoding.json"
        label_encoder_save_loc = f"{self.save_dir}/label_encoding.json"
        logging.info(f"Downloading {label_encoder_s3_loc}")
        self.download_file(s3_loc=label_encoder_s3_loc,
                           save_loc=label_encoder_save_loc)
        with open(f"{self.save_dir}/label_encoding.json") as f:
            self.label_encoding = json.load(f)

    def make_predictions_df(self, predictions):
        self.predictions_df = pd.DataFrame(
            list(
                zip(self.faces_files,
                    [self.label_encoding[str(x)] for x in predictions])))
        self.predictions_df.columns = ["face_file", "prediction"]
        self.predictions_df["video_name"] = self.video
        self.predictions_df["facenet_haar_model_name"] = self.run_name

    def query_ids(self):
        self.ids_df = pd.read_sql(f"""
            SELECT
                v.video_id,
                o.object_id,
                f.frame_number,
                o.object_cascade,
                o.object_x,
                o.object_y,
                o.object_w,
                o.object_h
            FROM videos v
            JOIN objects o
            ON v.video_id=o.video_id
            JOIN frames f
            ON o.frame_id=f.frame_id
            JOIN faces
            ON o.object_id=faces.object_id
            WHERE v.save_name = '{self.video}'
        """,
                                  con=self.engine)
        self.ids_df["face_file"] = self.ids_df.apply(lambda x: self.
                                                     make_file_names(x),
                                                     axis=1)

    def query_characters(self):
        self.characters_df = pd.read_sql(f"""
            SELECT v.series_id, c.character_id, REPLACE(c.character_name,' ', '_') AS character_name
            FROM videos v
            JOIN characters c
            ON v.series_id=c.series_id
            WHERE v.save_name = '{self.video}'
        """,
                                         con=self.engine)

    def make_file_names(self, d):
        result = f"f{d['frame_number']}c{d['object_cascade']}x{d['object_x']}y{d['object_y']}w{d['object_w']}h{d['object_h']}.jpg"
        return result

    def make_haar_facenet_predictions(self):
        self.haar_facenet_predictions = self.predictions_df.merge(
            self.ids_df).merge(self.characters_df,
                               left_on="prediction",
                               right_on="character_name")
        self.haar_facenet_predictions = self.haar_facenet_predictions[[
            "object_id", "video_id", "character_id", "facenet_haar_model_name"
        ]]

    def as_int(self, value):
        if value is None:
            return None
        else:
            return int(value)

    def create_record(self, row):
        if self.use_complete_tables:
            result = m.Facenet_Haar_Prediction_Complete(
                object_id=self.as_int(row["object_id"]),
                video_id=self.as_int(row["video_id"]),
                character_id=self.as_int(row["character_id"]),
                facenet_haar_model_name=row["facenet_haar_model_name"],
                timestamp_added=time())
        else:
            result = m.Facenet_Haar_Prediction(
                object_id=self.as_int(row["object_id"]),
                video_id=self.as_int(row["video_id"]),
                character_id=self.as_int(row["character_id"]),
                facenet_haar_model_name=row["facenet_haar_model_name"],
                timestamp_added=time())
        return result

    def predictions_to_sql(self, predictions):
        self.make_predictions_df(predictions)
        self.query_ids()
        self.query_characters()
        self.make_haar_facenet_predictions()
        # commit to pandas
        if self.use_complete_tables:
            self.hoodat_sql.commit_pandas_df(
                df=self.haar_facenet_predictions,
                table="haar_facenet_predictions_complete",
                record_func=self.create_record)
        else:
            self.hoodat_sql.commit_pandas_df(df=self.haar_facenet_predictions,
                                             table="haar_facenet_predictions",
                                             record_func=self.create_record)


# if __name__ == "__main__":
#     SERIES_NAME = sys.argv[1]
#     SAVE_DIR = sys.argv[2]
#     SQLALCHEMY_DATABASE_URI = "mysql://{0}:{1}@{2}/{3}".format(
#         env.DB_USER_RDS, env.DB_PASS_RDS, env.DB_SERVICE_RDS, env.DB_NAME_RDS
#     )
#     fnd = facenet_data(
#         series_name=SERIES_NAME,
#         save_dir=SAVE_DIR,
#         sqlalchemy_database_uri=SQLALCHEMY_DATABASE_URI)
#     fnd.execute()
#     logging.info("Script complete")



"""add_object_number_to_character_labels

Revision ID: 73cf781b10ad
Revises: e8ad754184fd
Create Date: 2017-10-29 11:20:52.544094

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "73cf781b10ad"
down_revision = "e8ad754184fd"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "character_labels", sa.Column("object_number", sa.Integer(), nullable=False)
    )
    op.create_unique_constraint(
        "object_label",
        "character_labels",
        ["video_id", "frame_number", "object_number"],
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("object_label", "character_labels", type_="unique")
    op.drop_column("character_labels", "object_number")
    # ### end Alembic commands ###



"""misspelt timestamp_add_process

Revision ID: 08a49ff86510
Revises: cc6544d0f3bb
Create Date: 2018-02-14 23:00:43.006672

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "08a49ff86510"
down_revision = "cc6544d0f3bb"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "processes",
        sa.Column("timestamp_add_process", sa.String(length=10), nullable=False),
    )
    op.drop_column("processes", "timestamp_add_processes")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "processes",
        sa.Column("timestamp_add_processes", mysql.VARCHAR(length=10), nullable=False),
    )
    op.drop_column("processes", "timestamp_add_process")
    # ### end Alembic commands ###



"""removed face tables and added to objects tables

Revision ID: d6ffb5f09ed6
Revises: 73cf781b10ad
Create Date: 2017-10-29 22:36:24.766985

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "d6ffb5f09ed6"
down_revision = "73cf781b10ad"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "objects",
        sa.Column("object_id", sa.Integer(), nullable=False),
        sa.Column("video_id", sa.Integer(), nullable=True),
        sa.Column("frame_id", sa.Integer(), nullable=True),
        sa.Column("episode_id", sa.Integer(), nullable=True),
        sa.Column("object_number", sa.Integer(), nullable=False),
        sa.Column("object_x", sa.Integer(), nullable=False),
        sa.Column("object_y", sa.Integer(), nullable=False),
        sa.Column("object_w", sa.Integer(), nullable=False),
        sa.Column("object_h", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["episode_id"],
            ["episodes.episode_id"],
        ),
        sa.ForeignKeyConstraint(
            ["frame_id"],
            ["frames.frame_id"],
        ),
        sa.ForeignKeyConstraint(
            ["video_id"],
            ["videos.video_id"],
        ),
        sa.PrimaryKeyConstraint("object_id"),
        sa.UniqueConstraint("video_id", "frame_id", "object_number", name="object"),
    )
    # op.execute("SET foreign_key_checks=0")
    # op.drop_table('faces')
    # op.execute("SET foreign_key_checks=1")
    # op.drop_table('face_classifications')
    # op.add_column('character_labels', sa.Column('character_label_id', sa.Integer(), nullable=False))
    # op.add_column('character_labels', sa.Column('object_id', sa.Integer(), nullable=True))
    # op.execute("ALTER TABLE character_labels DROP FOREIGN KEY character_labels_ibfk_2")
    # op.drop_constraint('character_labels_ibfk_2', 'character_labels', type_='foreignkey')
    op.drop_constraint("object_label", "character_labels", type_="unique")
    op.create_unique_constraint(
        "object_label", "character_labels", ["object_id", "character_id"]
    )
    op.create_foreign_key(
        None, "character_labels", "objects", ["object_id"], ["object_id"]
    )
    op.drop_column("character_labels", "object_x")
    op.drop_column("character_labels", "object_y")
    op.drop_column("character_labels", "object_number")
    op.drop_column("character_labels", "object_w")
    op.drop_column("character_labels", "object_h")
    op.drop_column("character_labels", "label_id")
    op.drop_column("character_labels", "frame_number")
    op.drop_column("character_labels", "video_id")
    op.create_unique_constraint("video_frame", "frames", ["video_id", "frame_number"])
    op.drop_constraint("frames_ibfk_2", "frames", type_="foreignkey")
    op.drop_column("frames", "user_id")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "frames",
        sa.Column(
            "user_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
    )
    op.create_foreign_key("frames_ibfk_2", "frames", "users", ["user_id"], ["id"])
    op.drop_constraint("video_frame", "frames", type_="unique")
    op.add_column(
        "character_labels",
        sa.Column(
            "video_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
    )
    op.add_column(
        "character_labels",
        sa.Column(
            "frame_number",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
    )
    op.add_column(
        "character_labels",
        sa.Column("label_id", mysql.INTEGER(display_width=11), nullable=False),
    )
    op.add_column(
        "character_labels",
        sa.Column(
            "object_h",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
    )
    op.add_column(
        "character_labels",
        sa.Column(
            "object_w",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
    )
    op.add_column(
        "character_labels",
        sa.Column(
            "object_number",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
    )
    op.add_column(
        "character_labels",
        sa.Column(
            "object_y",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
    )
    op.add_column(
        "character_labels",
        sa.Column(
            "object_x",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
    )
    op.drop_constraint(None, "character_labels", type_="foreignkey")
    op.create_foreign_key(
        "character_labels_ibfk_2",
        "character_labels",
        "videos",
        ["video_id"],
        ["video_id"],
    )
    op.drop_constraint("object_label", "character_labels", type_="unique")
    op.create_unique_constraint(
        "object_label",
        "character_labels",
        ["video_id", "frame_number", "object_number"],
    )
    op.drop_column("character_labels", "object_id")
    op.drop_column("character_labels", "character_label_id")
    op.create_table(
        "face_classifications",
        sa.Column(
            "face_classification_id", mysql.INTEGER(display_width=11), nullable=False
        ),
        sa.Column(
            "user_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "video_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "frame_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "face_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column("who", mysql.VARCHAR(length=128), nullable=False),
        sa.Column("confidence", mysql.FLOAT(), nullable=False),
        sa.Column(
            "timestamp_face_classified", mysql.VARCHAR(length=10), nullable=False
        ),
        sa.ForeignKeyConstraint(
            ["face_id"], ["faces.face_id"], name="face_classifications_ibfk_2"
        ),
        sa.ForeignKeyConstraint(
            ["frame_id"], ["frames.frame_id"], name="face_classifications_ibfk_1"
        ),
        sa.ForeignKeyConstraint(
            ["user_id"], ["users.id"], name="face_classifications_ibfk_4"
        ),
        sa.ForeignKeyConstraint(
            ["video_id"], ["videos.video_id"], name="face_classifications_ibfk_3"
        ),
        sa.PrimaryKeyConstraint("face_classification_id"),
        mysql_default_charset="utf8",
        mysql_engine="InnoDB",
    )
    op.create_table(
        "faces",
        sa.Column("face_id", mysql.INTEGER(display_width=11), nullable=False),
        sa.Column(
            "user_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "video_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "frame_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "face_x",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "face_y",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "face_w",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "face_h",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "episode_id",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
        sa.ForeignKeyConstraint(
            ["episode_id"], ["episodes.episode_id"], name="faces_ibfk_4"
        ),
        sa.ForeignKeyConstraint(["frame_id"], ["frames.frame_id"], name="faces_ibfk_3"),
        sa.ForeignKeyConstraint(["user_id"], ["users.id"], name="faces_ibfk_2"),
        sa.ForeignKeyConstraint(["video_id"], ["videos.video_id"], name="faces_ibfk_1"),
        sa.PrimaryKeyConstraint("face_id"),
        mysql_default_charset="utf8",
        mysql_engine="InnoDB",
    )
    op.drop_table("objects")
    # ### end Alembic commands ###



"""Linked videos with series

Revision ID: 11bf77399e0f
Revises: 7352cf06437d
Create Date: 2018-07-18 20:09:15.102202

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "11bf77399e0f"
down_revision = "7352cf06437d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("videos", sa.Column("series_id", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "videos", "series", ["series_id"], ["series_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "videos", type_="foreignkey")
    op.drop_column("videos", "series_id")
    # ### end Alembic commands ###



from collections import namedtuple


class Frame_Object():
    def __init__(self, object_record):
        self.record = dict(object_record)
        self.Rectangle_Tuple = namedtuple('Rectangle', 'xmin ymin xmax ymax')
        self.Rectangle = self.rectangle_from_record(object_record)

    def rectangle_from_record(self, object_record):
        return self.Rectangle_Tuple(
            object_record["object_x"], object_record["object_y"],
            object_record["object_x"] + object_record["object_w"],
            object_record["object_y"] + object_record["object_h"])

    # Methods related only to this object

    def area(self):
        """Calculate the area of the object in pixels"""
        a = self.Rectangle
        dx = a.xmax - a.xmin
        dy = a.ymax - a.ymin
        return dx * dy

    # Methods for comparing this object to others

    def contains(self, frame_object_b):
        """Check whether this object encompasses another object"""
        r1 = self.Rectangle
        r2 = frame_object_b.Rectangle
        return r1.xmin < r2.xmin < r2.xmax < r1.xmax and r1.ymin < r2.ymin < r2.ymax < r1.ymax

    def area_intersection(self, frame_object_b):
        a = self.Rectangle
        b = frame_object_b.Rectangle
        """Calculates area (in pixels) of intersection of two objects"""
        dx = min(a.xmax, b.xmax) - max(a.xmin, b.xmin)
        dy = min(a.ymax, b.ymax) - max(a.ymin, b.ymin)
        if (dx >= 0) and (dy >= 0):
            return dx * dy
        else:
            return 0

    def intersection_percentage(self, frame_object_b):
        # a = self.Rectangle
        # b = frame_object_b.Rectangle
        area_a = self.area()
        overlap = self.area_intersection(frame_object_b)
        return overlap / area_a



"""made episode name nullable

Revision ID: 8ff7642a7c59
Revises: 6906302f0e6e
Create Date: 2017-08-12 00:23:01.049392

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "8ff7642a7c59"
down_revision = "6906302f0e6e"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "episodes",
        "episode_name",
        existing_type=mysql.VARCHAR(length=128),
        nullable=True,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "episodes",
        "episode_name",
        existing_type=mysql.VARCHAR(length=128),
        nullable=False,
    )
    # ### end Alembic commands ###



"""Added boolean to videos indicating download process completion

Revision ID: 9133ec0c2db6
Revises: f54dfed2dafd
Create Date: 2017-08-08 19:34:12.935020

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9133ec0c2db6"
down_revision = "f54dfed2dafd"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "videos", sa.Column("download_process_complete", sa.Boolean(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("videos", "download_process_complete")
    # ### end Alembic commands ###



"""timestamp_adds not nullable and triggers uniqueness constraint

Revision ID: cc6544d0f3bb
Revises: 87163b759a86
Create Date: 2018-02-14 22:58:53.310498

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "cc6544d0f3bb"
down_revision = "87163b759a86"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "processes",
        "timestamp_add_processes",
        existing_type=mysql.VARCHAR(length=10),
        nullable=False,
    )
    op.alter_column(
        "triggers",
        "timestamp_add_trigger",
        existing_type=mysql.VARCHAR(length=10),
        nullable=False,
    )
    op.create_unique_constraint("video_process", "triggers", ["video_id", "process_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("video_process", "triggers", type_="unique")
    op.alter_column(
        "triggers",
        "timestamp_add_trigger",
        existing_type=mysql.VARCHAR(length=10),
        nullable=True,
    )
    op.alter_column(
        "processes",
        "timestamp_add_processes",
        existing_type=mysql.VARCHAR(length=10),
        nullable=True,
    )
    # ### end Alembic commands ###



"""
Functions for interacting with AWS S3
"""
#####################################################
#           Imports                                 #
#####################################################

import os
import boto3
import logging

#####################################################
#           Classes                                 #
#####################################################


class s3_data:
    def __init__(self, access_key, secret_key):
        self.access_key = access_key
        self.secret_key = secret_key
        self.client = boto3.client(
            "s3",
            aws_access_key_id=self.access_key,
            aws_secret_access_key=self.secret_key,
        )

    def list_dir(self, bucket, path, keys_only=True):
        paginator = self.client.get_paginator("list_objects")
        pageresponse = paginator.paginate(Bucket=bucket, Prefix=path, Delimiter="/")
        # PageResponse Holds 1000 objects at a time and will
        # continue to repeat in chunks of 1000.
        file_list = []
        for pageobject in pageresponse:
            for file in pageobject["Contents"]:
                if keys_only:
                    file_list.append(file["Key"])
                else:
                    file_list.append(file)
        return file_list

    def download_dir(self, region, bucket, s3_loc, save_loc):
        resource = boto3.resource(
            "s3",
            aws_access_key_id=self.access_key,
            aws_secret_access_key=self.secret_key,
            region_name=region,
        )
        paginator = self.client.get_paginator("list_objects")
        pageresponse = paginator.paginate(Bucket=bucket, Prefix=s3_loc, Delimiter="/")
        logging.info("checkpoint a")
        for pageobject in pageresponse:
            logging.info("checkpoint a2")
            if pageobject.get("CommonPrefixes") is not None:
                logging.info("checkpoint b")
                for subdir in pageobject.get("CommonPrefixes"):
                    self.download_dir(
                        # client, resource, subdir.get('Prefix'), save_loc, bucket
                        region,
                        bucket,
                        s3_loc,
                        save_loc,
                    )
            if pageobject.get("Contents") is not None:
                logging.info("checkpoint c")
                for file in pageobject.get("Contents"):
                    file_name = file.get("Key").replace(s3_loc, "")
                    save_path = os.path.normpath(save_loc) + os.sep + file_name
                    save_dir = os.sep.join(save_path.split(os.sep)[:-1])
                    if not os.path.exists(save_dir):
                        os.makedirs(save_dir)
                    resource.meta.client.download_file(
                        bucket, file.get("Key"), save_path
                    )

    # def upload_dir(self, bucket, local_dir, s3_loc):
    #     # enumerate local files recursively
    #     for root, dirs, files in os.walk(local_dir):
    #         for filename in files:
    #             # construct the full local path
    #             local_path = os.path.join(root, filename)
    #             # construct the full s3 path
    #             relative_path = os.path.relpath(local_path, local_dir)
    #             s3_path = os.path.join(s3_loc, relative_path)
    #             logging.info('Searching {0} in {1}'.format(s3_path, bucket))
    #             try:
    #                 self.client.head_object(Bucket=bucket, Key=s3_path)
    #                 logging.info(
    #                     "Path already found on S3. Skipping {0}...".format(
    #                         s3_path))
    #             except:
    #                 logging.info("Uploading {0}...".format(s3_path))
    #                 self.client.upload_file(local_path, bucket, s3_path)
    #     logging.info("Upload complete")

    def download_file(self, bucket, s3_loc, save_loc):
        file_list = self.client.download_file(bucket, s3_loc, save_loc)
        return file_list

    def upload_file(self, bucket, s3_loc, local_loc):
        file_list = self.client.upload_file(local_loc, bucket, s3_loc)
        return file_list



"""Added seasons table

Revision ID: 60a827c82297
Revises: 6c12ddacf663
Create Date: 2017-08-11 19:43:01.130858

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "60a827c82297"
down_revision = "6c12ddacf663"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "seasons",
        sa.Column("season_id", sa.Integer(), nullable=False),
        sa.Column("season_number", sa.Integer(), nullable=False),
        sa.Column("season_year_start", sa.Integer(), nullable=False),
        sa.Column("series_id", sa.Integer(), nullable=True),
        sa.Column("timestamp_add_season", sa.String(length=10), nullable=False),
        sa.ForeignKeyConstraint(
            ["series_id"],
            ["series.series_id"],
        ),
        sa.PrimaryKeyConstraint("season_id"),
    )
    op.add_column("episodes", sa.Column("episode_number", sa.Integer(), nullable=False))
    op.add_column("episodes", sa.Column("season_id", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "episodes", "seasons", ["season_id"], ["season_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "episodes", type_="foreignkey")
    op.drop_column("episodes", "season_id")
    op.drop_column("episodes", "episode_number")
    op.drop_table("seasons")
    # ### end Alembic commands ###



"""removed character_labels relationship from videos table

Revision ID: 1263b617af57
Revises: d6ffb5f09ed6
Create Date: 2017-10-30 19:23:09.058365

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "1263b617af57"
down_revision = "d6ffb5f09ed6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""added dlib face tables

Revision ID: c56cae55295e
Revises: 2a3a19b776c5
Create Date: 2020-06-03 13:53:22.656333

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c56cae55295e"
down_revision = "2a3a19b776c5"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""test revision

Revision ID: 63fcb6f73f50
Revises:
Create Date: 2017-07-28 23:55:13.910104

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "63fcb6f73f50"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    pass


def downgrade():
    pass



"""Linked videos and actor_groups

Revision ID: 7352cf06437d
Revises: 8f5ca94205dc
Create Date: 2018-07-18 19:26:38.558941

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "7352cf06437d"
down_revision = "8f5ca94205dc"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("videos", sa.Column("actor_group_id", sa.Integer(), nullable=True))
    op.create_foreign_key(
        None, "videos", "actor_groups", ["actor_group_id"], ["actor_group_id"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "videos", type_="foreignkey")
    op.drop_column("videos", "actor_group_id")
    # ### end Alembic commands ###



"""Added uniqueness to youtube_link column

Revision ID: 522cd0c45168
Revises: 55b537e0c4db
Create Date: 2017-07-30 11:29:48.582700

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "522cd0c45168"
down_revision = "55b537e0c4db"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(None, "videos", ["youtube_link"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "videos", type_="unique")
    # ### end Alembic commands ###



"""fixed error in episode faces relationship again

Revision ID: f54dfed2dafd
Revises: 9a3b4cac9346
Create Date: 2017-08-07 22:19:58.298629

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "f54dfed2dafd"
down_revision = "9a3b4cac9346"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""added episodes table

Revision ID: e1d164c37112
Revises: 554178515c27
Create Date: 2017-08-07 21:31:35.799509

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "e1d164c37112"
down_revision = "554178515c27"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "episodes",
        sa.Column("episode_id", sa.Integer(), nullable=False),
        sa.Column("episode_name", sa.String(length=128), nullable=False),
        sa.Column("episode_date", sa.String(length=10), nullable=True),
        sa.Column("timestamp_add_episode", sa.String(length=10), nullable=False),
        sa.Column("series_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["series_id"],
            ["series.series_id"],
        ),
        sa.PrimaryKeyConstraint("episode_id"),
    )
    op.add_column("faces", sa.Column("episode_id", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "faces", "episodes", ["episode_id"], ["episode_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "faces", type_="foreignkey")
    op.drop_column("faces", "episode_id")
    op.drop_table("episodes")
    # ### end Alembic commands ###



"""some uniqueness constraints

Revision ID: 6906302f0e6e
Revises: 52cd9c67142d
Create Date: 2017-08-11 20:11:05.347454

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "6906302f0e6e"
down_revision = "52cd9c67142d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "series_character", "characters", ["series_id", "character_name"]
    )
    op.create_unique_constraint(
        "season_episode", "episodes", ["episode_number", "season_id"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("season_episode", "episodes", type_="unique")
    op.drop_constraint("series_character", "characters", type_="unique")
    # ### end Alembic commands ###



"""fixed error in episode faces relationship

Revision ID: 9a3b4cac9346
Revises: e1d164c37112
Create Date: 2017-08-07 21:33:34.865387

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9a3b4cac9346"
down_revision = "e1d164c37112"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""unique constraint in seasons

Revision ID: 52cd9c67142d
Revises: 60a827c82297
Create Date: 2017-08-11 19:52:45.226185

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "52cd9c67142d"
down_revision = "60a827c82297"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "series_season", "seasons", ["series_id", "season_number"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("series_season", "seasons", type_="unique")
    # ### end Alembic commands ###



# Base Imports
import logging

# Libraries
import pandas as pd

# SqlAlchemy Imports
import sqlalchemy
from sqlalchemy import text
from sqlalchemy.orm import Session
from sqlalchemy.exc import IntegrityError


import os
import pymysql


class HoodatSqlalchemy:
    def __init__(self, sqlalchemy_database_uri):
        self.sqlalchemy_database_uri = sqlalchemy_database_uri
        self.engine = sqlalchemy.create_engine(self.sqlalchemy_database_uri)

    def create_session(self):
        self.session = Session(self.engine)
        return self.session

    def commit_pandas_df(self, df, table, record_func):
        session = self.create_session()
        logging.info(f"Populating {table} table")
        n_rows = df.shape[0]
        logging.info(f"Total rows: {n_rows}")
        for i in range(n_rows):
            x = df.iloc[i]
            try:
                record = record_func(x)
                session.add(record)
                session.flush()
            except IntegrityError as e:
                logging.info("Record already exists in db")
                logging.info(e)
                session.rollback()
            else:
                logging.info("Adding row")
                session.commit()
        session.close()

    def query(self, query: str):
        """Run a query against the database

        Args:
            query (str): A SQL query to run
        """
        session = self.create_session()
        session.execute(text(query))
        session.close()

    def query_to_df(self, query):
        """Run a query against the database and return a pandas dataframe

        Args:
            query (str): A SQL query to run
        """
        session = self.create_session()
        query_result = session.execute(text(query)).all()
        session.close()
        df = pd.DataFrame(query_result)
        return df

    def create_views(self, views_obj: object, overwrite: bool = False):
        """Creates views defined in an object

        Args:
            views_obj (object): An object with `key: value` being `name of view: query to create view`
            overwrite (boolean) = False: Whether or not to overwrite existing views
        """
        for view_key in views_obj.keys():
            if overwrite:
                print("Overwrite is true")
                prefix = "CREATE OR REPLACE VIEW"
            else:
                prefix = "CREATE VIEW"
            try:
                session = self.create_session()
                query = f"{prefix} `{view_key}` AS {views_obj[view_key]}"
                session.execute(text(query))
                session.close()
            except Exception as e:
                return f"Failed to create views with error: {e}"
        return "Success"



"""Added process_order to processes table

Revision ID: 2a3a19b776c5
Revises: aa0ccca01f54
Create Date: 2020-05-28 16:56:35.636004

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "2a3a19b776c5"
down_revision = "aa0ccca01f54"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("haar_faces_deduplicated")
    op.add_column("processes", sa.Column("process_order", sa.Integer(), nullable=True))
    op.create_unique_constraint(None, "processes", ["airflow_dag_id"])
    op.create_unique_constraint(None, "processes", ["process_name"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "processes", type_="unique")
    op.drop_constraint(None, "processes", type_="unique")
    op.drop_column("processes", "process_order")
    op.create_table(
        "haar_faces_deduplicated",
        sa.Column(
            "object_id",
            mysql.BIGINT(display_width=20),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "video_id",
            mysql.BIGINT(display_width=20),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "frame_id",
            mysql.BIGINT(display_width=20),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column("episode_id", mysql.TEXT(), nullable=True),
        sa.Column(
            "object_number",
            mysql.BIGINT(display_width=20),
            autoincrement=False,
            nullable=True,
        ),
        mysql_default_charset="latin1",
        mysql_engine="InnoDB",
    )
    # ### end Alembic commands ###



"""Make series_name column a string column

Revision ID: 8fd3900c2c9e
Revises: bc0978602558
Create Date: 2017-07-29 12:35:15.558688

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "8fd3900c2c9e"
down_revision = "bc0978602558"
branch_labels = None
depends_on = None


def upgrade():
    op.execute("ALTER TABLE series MODIFY series_name VARCHAR(128) NOT NULL")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""add columns to actors autogen

Revision ID: ae93ed2eb8f0
Revises: 1b019da65d4f
Create Date: 2017-07-29 17:36:19.692127

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "ae93ed2eb8f0"
down_revision = "1b019da65d4f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "actors", sa.Column("actor_name", sa.String(length=128), nullable=False)
    )
    op.add_column("characters", sa.Column("actor_id", sa.Integer(), nullable=True))
    op.add_column(
        "characters", sa.Column("character_name", sa.String(length=128), nullable=False)
    )
    op.add_column("characters", sa.Column("series_id", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "characters", "series", ["series_id"], ["series_id"])
    op.create_foreign_key(None, "characters", "actors", ["actor_id"], ["actor_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "characters", type_="foreignkey")
    op.drop_constraint(None, "characters", type_="foreignkey")
    op.drop_column("characters", "series_id")
    op.drop_column("characters", "character_name")
    op.drop_column("characters", "actor_id")
    op.drop_column("actors", "actor_name")
    # ### end Alembic commands ###



"""add object_pca_features table

Revision ID: 4fdaad670a35
Revises: c4d469256008
Create Date: 2018-07-05 08:20:04.822029

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "4fdaad670a35"
down_revision = "c4d469256008"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "object_pca_features",
        sa.Column("pca_features_id", sa.Integer(), nullable=False),
        sa.Column("object_id", sa.Integer(), nullable=True),
        sa.Column("feature_1", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_2", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_3", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_4", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_5", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_6", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_7", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_8", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_9", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_10", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_11", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_12", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_13", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_14", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_15", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_16", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_17", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_18", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_19", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_20", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_21", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_22", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_23", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_24", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_25", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_26", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_27", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_28", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_29", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_30", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_31", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.Column("feature_32", sa.Numeric(precision=7, scale=6), nullable=False),
        sa.ForeignKeyConstraint(
            ["object_id"],
            ["objects.object_id"],
        ),
        sa.PrimaryKeyConstraint("pca_features_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("object_pca_features")
    # ### end Alembic commands ###



"""add timestamps added for processes and triggers

Revision ID: 87163b759a86
Revises: 499e890aa148
Create Date: 2018-02-14 22:49:22.775146

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "87163b759a86"
down_revision = "499e890aa148"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "processes",
        sa.Column("timestamp_add_processes", sa.String(length=10), nullable=True),
    )
    op.add_column(
        "triggers",
        sa.Column("timestamp_add_trigger", sa.String(length=10), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("triggers", "timestamp_add_trigger")
    op.drop_column("processes", "timestamp_add_processes")
    # ### end Alembic commands ###



"""add labels_table

Revision ID: e8ad754184fd
Revises: b587da5794d6
Create Date: 2017-10-29 10:00:27.232737

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "e8ad754184fd"
down_revision = "b587da5794d6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "character_labels",
        sa.Column("label_id", sa.Integer(), nullable=False),
        sa.Column("video_id", sa.Integer(), nullable=True),
        sa.Column("frame_number", sa.Integer(), nullable=False),
        sa.Column("object_x", sa.Integer(), nullable=False),
        sa.Column("object_y", sa.Integer(), nullable=False),
        sa.Column("object_w", sa.Integer(), nullable=False),
        sa.Column("object_h", sa.Integer(), nullable=False),
        sa.Column("character_id", sa.Integer(), nullable=True),
        sa.Column("timestamp_labelled", sa.String(length=10), nullable=False),
        sa.ForeignKeyConstraint(
            ["character_id"],
            ["characters.character_id"],
        ),
        sa.ForeignKeyConstraint(
            ["video_id"],
            ["videos.video_id"],
        ),
        sa.PrimaryKeyConstraint("label_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("character_labels")
    # ### end Alembic commands ###



"""Added video_type column, made character foreign keys not nullable

Revision ID: d51287a30494
Revises: ae93ed2eb8f0
Create Date: 2017-07-30 10:17:12.878851

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "d51287a30494"
down_revision = "ae93ed2eb8f0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "characters",
        "actor_id",
        existing_type=mysql.INTEGER(display_width=11),
        nullable=False,
    )
    op.alter_column(
        "characters",
        "series_id",
        existing_type=mysql.INTEGER(display_width=11),
        nullable=False,
    )
    op.add_column(
        "videos", sa.Column("video_type", sa.String(length=48), nullable=False)
    )
    op.drop_index("video_display_name", table_name="videos")
    op.drop_column("videos", "video_display_name")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "videos",
        sa.Column("video_display_name", mysql.VARCHAR(length=156), nullable=False),
    )
    op.create_index("video_display_name", "videos", ["video_display_name"], unique=True)
    op.drop_column("videos", "video_type")
    op.alter_column(
        "characters",
        "series_id",
        existing_type=mysql.INTEGER(display_width=11),
        nullable=True,
    )
    op.alter_column(
        "characters",
        "actor_id",
        existing_type=mysql.INTEGER(display_width=11),
        nullable=True,
    )
    # ### end Alembic commands ###



"""
Functions for interacting with GCP cloud storage
"""
#####################################################
#           Imports                                 #
#####################################################

# import os
from google.cloud import storage
# import logging

#####################################################
#           Classes                                 #
#####################################################


class gs():
    def __init__(self, creds_file: str, creds_info: str) -> None:
        if creds_file is not None:
            self.client = storage.Client().from_service_account_json(creds_file)
        if creds_info is not None:
            self.client = storage.Client().from_service_account_info(creds_info)

    def check_file_exists(self, bucket_name: str, name: str):
        bucket = self.client.bucket(bucket_name)
        stats = storage.Blob(bucket=bucket, name=name).exists(self.client)
        return stats

    def list_dir(self, bucket_name, path, keys_only=True):
        blobs = self.client.list_blobs(
            bucket_name,
            prefix=path,
            delimiter=None)
        blob_names = [blob.name for blob in blobs]
        if keys_only:
            blob_names = [blob_name.split("/")[-1] for blob_name in blob_names]
        return blob_names

    def download_dir(self, region, bucket_name, s3_loc, save_loc):
        pass

    def upload_dir(self, bucket_name, local_dir, gs_loc):
        pass

    def remove_file(self, bucket_name, gs_loc):
        bucket = self.client.get_bucket(bucket_name)
        blob = bucket.blob(gs_loc)
        blob.delete()

    def download_file(self, bucket_name, gs_loc, save_loc):
        bucket = self.client.get_bucket(bucket_name)
        blob = bucket.get_blob(gs_loc)
        blob.download_to_filename(save_loc)

    def upload_file(self, bucket_name, gs_loc, local_loc) -> None:
        bucket = self.client.get_bucket(bucket_name)
        blob = bucket.blob(gs_loc)
        blob.upload_from_filename(local_loc)



"""character_labels needs a key col

Revision ID: 27c3ed745239
Revises: 817384c566f4
Create Date: 2017-10-31 21:05:49.454699

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "27c3ed745239"
down_revision = "817384c566f4"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hoodat_utils', 'hoodat_utils.alembic', 'hoodat_utils.alembic.versions']

package_data = \
{'': ['*']}

install_requires = \
['Flask-Login>=0.6.0,<0.7.0',
 'Flask-SQLAlchemy>=2.5.1,<3.0.0',
 'PyMySQL>=1.0.2,<2.0.0',
 'SQLAlchemy>=1.4.35,<2.0.0',
 'alembic>=1.8.1,<2.0.0',
 'google-cloud-storage>=1.20.0,<2.0.0',
 'pandas>=1.4.2,<2.0.0',
 'pytest>=7.1.1,<8.0.0']

setup_kwargs = {
    'name': 'hoodat-utils',
    'version': '0.0.9',
    'description': '',
    'long_description': None,
    'author': 'Eugene Brown',
    'author_email': 'efbbrown@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Added save_name to videos table

Revision ID: a25b2b88671d
Revises: 522cd0c45168
Create Date: 2017-07-30 13:38:57.668644

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "a25b2b88671d"
down_revision = "522cd0c45168"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "videos", sa.Column("save_name", sa.String(length=156), nullable=False)
    )
    op.create_unique_constraint(None, "videos", ["save_name"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "videos", type_="unique")
    op.drop_column("videos", "save_name")
    # ### end Alembic commands ###



"""added processes and triggers tables

Revision ID: 499e890aa148
Revises: babc9bcf7b14
Create Date: 2018-02-11 20:29:01.811394

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "499e890aa148"
down_revision = "babc9bcf7b14"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "processes",
        sa.Column("process_id", sa.Integer(), nullable=False),
        sa.Column("process_name", sa.String(length=32), nullable=False),
        sa.Column("airflow_dag_id", sa.String(length=32), nullable=False),
        sa.PrimaryKeyConstraint("process_id"),
    )
    op.create_table(
        "triggers",
        sa.Column("trigger_id", sa.Integer(), nullable=False),
        sa.Column("video_id", sa.Integer(), nullable=False),
        sa.Column("process_id", sa.Integer(), nullable=False),
        sa.Column("status_complete", sa.Boolean(), nullable=False),
        sa.ForeignKeyConstraint(
            ["process_id"],
            ["processes.process_id"],
        ),
        sa.ForeignKeyConstraint(
            ["video_id"],
            ["videos.video_id"],
        ),
        sa.PrimaryKeyConstraint("trigger_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("triggers")
    op.drop_table("processes")
    # ### end Alembic commands ###



"""added cascade column to objects table

Revision ID: 817384c566f4
Revises: 1263b617af57
Create Date: 2017-10-30 22:14:39.428483

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "817384c566f4"
down_revision = "1263b617af57"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("objects", sa.Column("object_cascade", sa.Integer(), nullable=False))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("objects", "object_cascade")
    # ### end Alembic commands ###



"""make character_label_ids a primary key

Revision ID: babc9bcf7b14
Revises: 27c3ed745239
Create Date: 2017-10-31 21:14:03.926580

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "babc9bcf7b14"
down_revision = "27c3ed745239"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("character_labels", "character_label_id")
    op.execute(
        "ALTER TABLE character_labels ADD character_label_id INT PRIMARY KEY AUTO_INCREMENT;"
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("character_labels", "character_label_id")
    op.execute("ALTER TABLE character_labels ADD character_label_id INT;")
    # ### end Alembic commands ###




# Library Imports
# import pandas as pd
# from sqlalchemy.orm import Session
from sqlalchemy import create_engine

# Local Imports
# from hoodat_utils import gs
from hoodat_utils import models as m


class create_dataset():

    def __init__(self, series_name, sqlalchemy_database_uri):
        self.series_name = series_name
        self.sqlalchemy_database_uri = sqlalchemy_database_uri
        self.engine = create_engine(self.sqlalchemy_database_uri)

    def query_series_character_labels(self):
        query = self.session.query(
            m.Series.series_id, m.Video.video_id, m.Video.video_type, m.Video.save_name, m.Frame.frame_id,
            m.Frame.frame_number, m.Object.object_id,
            m.Character_Label.character_label_id, m.Object.object_x,
            m.Object.object_y, m.Object.object_w, m.Object.object_h,
            m.Object.object_cascade, m.Character.character_name
        ).filter(
            m.Series.series_name == self.series_name
        ).join(m.Episode).join(m.Video).join(m.Frame).join(m.Object).join(m.Character_Label).join(m.Character)
        series_character_labels = query.all()
        print("Number of labels: {0}".format(str(len(series_character_labels))))
        return series_character_labels



#####################################################
#           Imports                                 #
#####################################################

from flask_sqlalchemy import SQLAlchemy
from flask_login import UserMixin
from sqlalchemy.orm import backref

#####################################################
#           Script                                  #
#####################################################

db = SQLAlchemy()


class User(db.Model, UserMixin):

    __tablename__ = "users"

    id = db.Column(db.Integer, primary_key=True)
    social_id = db.Column(db.String(64), nullable=False, unique=True)
    full_name = db.Column(db.String(64))
    display_name = db.Column(db.String(48))
    signup_timestamp = db.Column(db.String(10), nullable=False)
    videos = db.relationship("Video", backref="User", lazy="dynamic")

    def __repr__(self):
        return "<User %r>" % (self.display_name)


class Video(db.Model):

    __tablename__ = "videos"

    video_id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey("users.id"), nullable=False)
    episode_id = db.Column(db.Integer, db.ForeignKey("episodes.episode_id"))
    video_name = db.Column(db.String(156), nullable=False, unique=True)
    save_name = db.Column(db.String(156), nullable=False, unique=True)
    video_type = db.Column(db.String(48), nullable=False)
    youtube_link = db.Column(db.String(128), unique=True)
    timestamp_add_video = db.Column(db.String(10), nullable=False)
    download_process_complete = db.Column(db.Boolean)
    series_id = db.Column(db.Integer, db.ForeignKey("series.series_id"))
    actor_group_id = db.Column(db.Integer, db.ForeignKey("actor_groups.actor_group_id"))
    frames = db.relationship("Frame", backref="Video", lazy="dynamic")
    frames_complete = db.relationship("Frame_Complete", backref="Video", lazy="dynamic")
    objects = db.relationship("Object", backref="Video", lazy="dynamic")
    objects_complete = db.relationship(
        "Object_Complete", backref="Video", lazy="dynamic"
    )
    triggers = db.relationship("Trigger", backref="Video", lazy="dynamic")
    faces = db.relationship("Face", backref="Video", lazy="dynamic")
    facenet_haar_predictions = db.relationship(
        "Facenet_Haar_Prediction", backref=backref("Video", uselist=False)
    )

    __table_args__ = (
        db.UniqueConstraint("video_id", "episode_id", name="episode_video"),
    )


class Frame(db.Model):

    __tablename__ = "frames"

    frame_id = db.Column(db.Integer, primary_key=True)
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_number = db.Column(db.Integer, nullable=False)
    objects = db.relationship("Object", backref="Frame", lazy="dynamic")
    face = db.relationship(
        "Face", backref=backref("Frame", uselist=False), lazy="dynamic"
    )

    __table_args__ = (
        db.UniqueConstraint("video_id", "frame_number", name="video_frame"),
    )


class Frame_Complete(db.Model):

    __tablename__ = "frames_complete"

    frame_id = db.Column(db.Integer, primary_key=True)
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_number = db.Column(db.Integer, nullable=False)
    objects_complete = db.relationship(
        "Object_Complete", backref="Frame_Complete", lazy="dynamic"
    )
    faces_complete = db.relationship(
        "Face_Complete",
        backref=backref("Frame_Complete", uselist=False),
        lazy="dynamic",
    )

    __table_args__ = (
        db.UniqueConstraint("video_id", "frame_number", name="video_frame"),
    )


class Object(db.Model):

    __tablename__ = "objects"

    object_id = db.Column(db.Integer, primary_key=True)
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_id = db.Column(db.Integer, db.ForeignKey("frames.frame_id"))
    episode_id = db.Column(db.Integer, db.ForeignKey("episodes.episode_id"))
    object_number = db.Column(db.Integer, nullable=False)
    object_cascade = db.Column(db.Integer, nullable=False)
    object_x = db.Column(db.Integer, nullable=False)
    object_y = db.Column(db.Integer, nullable=False)
    object_w = db.Column(db.Integer, nullable=False)
    object_h = db.Column(db.Integer, nullable=False)
    character_label = db.relationship(
        "Character_Label", backref=backref("Object", uselist=False)
    )
    object_pca_features = db.relationship(
        "Object_PCA_Features", backref=backref("Object", uselist=False)
    )
    object_umap_features = db.relationship(
        "Object_UMAP_Features", backref=backref("Object", uselist=False)
    )
    face = db.relationship("Face", backref=backref("Object", uselist=False))
    facenet_haar_predictions = db.relationship(
        "Facenet_Haar_Prediction", backref=backref("Object", uselist=False)
    )

    __table_args__ = (
        db.UniqueConstraint("video_id", "frame_id", "object_number", name="object"),
    )


class Object_Complete(db.Model):

    __tablename__ = "objects_complete"

    object_id = db.Column(db.Integer, primary_key=True)
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_id = db.Column(db.Integer, db.ForeignKey("frames_complete.frame_id"))
    episode_id = db.Column(db.Integer, db.ForeignKey("episodes.episode_id"))
    object_number = db.Column(db.Integer, nullable=False)
    object_cascade = db.Column(db.Integer, nullable=False)
    object_x = db.Column(db.Integer, nullable=False)
    object_y = db.Column(db.Integer, nullable=False)
    object_w = db.Column(db.Integer, nullable=False)
    object_h = db.Column(db.Integer, nullable=False)
    faces_complete = db.relationship(
        "Face_Complete",
        backref=backref("Object_Complete", uselist=False),
        lazy="dynamic",
    )
    facenet_haar_predictions = db.relationship(
        "Facenet_Haar_Prediction", backref=backref("Object_Complete", uselist=False)
    )
    # character_label = db.relationship(
    #     "Character_Label", backref=backref("Object", uselist=False))
    # object_pca_features = db.relationship(
    #     "Object_PCA_Features", backref=backref("Object", uselist=False))
    # object_umap_features = db.relationship(
    #     "Object_UMAP_Features", backref=backref("Object", uselist=False))
    # face = db.relationship(
    #     "Face", backref=backref("Object", uselist=False))
    # facenet_haar_predictions = db.relationship(
    #     "Facenet_Haar_Prediction", backref=backref("Object", uselist=False))

    __table_args__ = (
        db.UniqueConstraint("video_id", "frame_id", "object_number", name="object"),
    )


class Face(db.Model):

    __tablename__ = "faces"

    face_id = db.Column(db.Integer, primary_key=True)
    object_id = db.Column(db.Integer, db.ForeignKey("objects.object_id"))
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_id = db.Column(db.Integer, db.ForeignKey("frames.frame_id"))
    episode_id = db.Column(db.Integer, db.ForeignKey("episodes.episode_id"))
    object_number = db.Column(db.Integer, nullable=False)

    __table_args__ = (
        db.UniqueConstraint("video_id", "frame_id", "object_number", name="object"),
    )


class Face_Complete(db.Model):

    __tablename__ = "faces_complete"

    face_id = db.Column(db.Integer, primary_key=True)
    object_id = db.Column(db.Integer, db.ForeignKey("objects_complete.object_id"))
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_id = db.Column(db.Integer, db.ForeignKey("frames_complete.frame_id"))
    episode_id = db.Column(db.Integer, db.ForeignKey("episodes.episode_id"))
    object_number = db.Column(db.Integer, nullable=False)

    __table_args__ = (
        db.UniqueConstraint("video_id", "frame_id", "object_number", name="object"),
    )


class Dlib_Face_Unaligned(db.Model):

    __tablename__ = "dlib_faces_unaligned"

    dlib_faces_unaligned_id = db.Column(db.Integer, primary_key=True)
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_id = db.Column(db.Integer, db.ForeignKey("frames.frame_id"))
    episode_id = db.Column(db.Integer, db.ForeignKey("episodes.episode_id"))
    object_number = db.Column(db.Integer, nullable=False)
    object_x = db.Column(db.Integer, nullable=False)
    object_y = db.Column(db.Integer, nullable=False)
    object_w = db.Column(db.Integer, nullable=False)
    object_h = db.Column(db.Integer, nullable=False)
    # character_label = db.relationship("Character_Label", backref=backref("Dlib_Face_Unaligned", uselist=False))

    __table_args__ = (
        db.UniqueConstraint(
            "video_id", "frame_id", "object_number", name="dlib_face_unaligned"
        ),
    )


class Dlib_Face_Aligned(db.Model):

    __tablename__ = "dlib_faces_aligned"

    dlib_faces_aligned_id = db.Column(db.Integer, primary_key=True)
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"))
    frame_id = db.Column(db.Integer, db.ForeignKey("frames.frame_id"))
    episode_id = db.Column(db.Integer, db.ForeignKey("episodes.episode_id"))
    object_number = db.Column(db.Integer, nullable=False)
    object_x = db.Column(db.Integer, nullable=False)
    object_y = db.Column(db.Integer, nullable=False)
    object_w = db.Column(db.Integer, nullable=False)
    object_h = db.Column(db.Integer, nullable=False)
    # character_label = db.relationship("Character_Label", backref=backref("Dlib_Face_Aligned", uselist=False))

    __table_args__ = (
        db.UniqueConstraint(
            "video_id", "frame_id", "object_number", name="dlib_faces_aligned"
        ),
    )


class Character_Label(db.Model):

    __tablename__ = "character_labels"

    character_label_id = db.Column(db.Integer, primary_key=True)
    object_id = db.Column(db.Integer, db.ForeignKey("objects.object_id"))
    character_id = db.Column(db.Integer, db.ForeignKey("characters.character_id"))
    timestamp_labelled = db.Column(db.String(10), nullable=False)

    __table_args__ = (
        db.UniqueConstraint("object_id", "character_id", name="object_label"),
    )


class Series(db.Model):

    __tablename__ = "series"

    series_id = db.Column(db.Integer, primary_key=True)
    series_name = db.Column(db.String(128), nullable=False)
    series_release_year = db.Column(db.Integer)
    series_end_year = db.Column(db.Integer)
    timestamp_add_series = db.Column(db.String(10), nullable=False)
    videos = db.relationship("Video", backref="Series", lazy="dynamic")
    characters = db.relationship("Character", backref="Series", lazy="dynamic")
    seasons = db.relationship("Season", backref="Series", lazy="dynamic")
    episodes = db.relationship("Episode", backref="Series", lazy="dynamic")


class Season(db.Model):

    __tablename__ = "seasons"

    season_id = db.Column(db.Integer, primary_key=True)
    season_number = db.Column(db.Integer, nullable=False)
    season_year_start = db.Column(db.Integer, nullable=False)
    series_id = db.Column(db.Integer, db.ForeignKey("series.series_id"))
    timestamp_add_season = db.Column(db.String(10), nullable=False)

    __table_args__ = (
        db.UniqueConstraint("series_id", "season_number", name="series_season"),
    )


class Episode(db.Model):

    __tablename__ = "episodes"

    episode_id = db.Column(db.Integer, primary_key=True)
    episode_name = db.Column(db.String(128))
    episode_number = db.Column(db.Integer, nullable=False)
    episode_date = db.Column(db.String(10))
    timestamp_add_episode = db.Column(db.String(10), nullable=False)
    series_id = db.Column(db.Integer, db.ForeignKey("series.series_id"))
    season_id = db.Column(db.Integer, db.ForeignKey("seasons.season_id"))
    video = db.relationship("Video", backref="Episode", uselist=False)
    face = db.relationship(
        "Face", backref=backref("Episode", uselist=False), lazy="dynamic"
    )

    __table_args__ = (
        db.UniqueConstraint("episode_number", "season_id", name="season_episode"),
    )


class Actor(db.Model):

    __tablename__ = "actors"

    actor_id = db.Column(db.Integer, primary_key=True)
    actor_name = db.Column(db.String(128), nullable=False)
    timestamp_add_actor = db.Column(db.String(10), nullable=False)
    characters = db.relationship("Character", backref="Actor", lazy="dynamic")
    actor_group_lookups = db.relationship(
        "Actor_Group_Lookup", backref="Actor", lazy="dynamic"
    )


class Actor_Group(db.Model):

    __tablename__ = "actor_groups"

    actor_group_id = db.Column(db.Integer, primary_key=True)
    actor_group_name = db.Column(db.String(64), nullable=False)
    timestamp_add_actor_group = db.Column(db.String(10), nullable=False)
    actor_group_lookups = db.relationship(
        "Actor_Group_Lookup", backref="Actor_Group", lazy="dynamic"
    )
    videos = db.relationship("Video", backref="Actor_Group", lazy="dynamic")


class Actor_Group_Lookup(db.Model):

    __tablename__ = "actor_group_lookup"

    actor_group_lookup_id = db.Column(db.Integer, primary_key=True)
    actor_group_id = db.Column(
        db.Integer, db.ForeignKey("actor_groups.actor_group_id"), nullable=False
    )
    actor_id = db.Column(db.Integer, db.ForeignKey("actors.actor_id"), nullable=False)
    timestamp_add_actor_group_lookup = db.Column(db.String(10), nullable=False)


class Character(db.Model):

    __tablename__ = "characters"

    character_id = db.Column(db.Integer, primary_key=True)
    series_id = db.Column(db.Integer, db.ForeignKey("series.series_id"), nullable=False)
    actor_id = db.Column(db.Integer, db.ForeignKey("actors.actor_id"), nullable=False)
    character_name = db.Column(db.String(128), nullable=False)
    timestamp_add_character = db.Column(db.String(10), nullable=False)
    character_labels = db.relationship(
        "Character_Label", backref="Character", lazy="dynamic"
    )
    facenet_haar_predictions = db.relationship(
        "Facenet_Haar_Prediction", backref=backref("Character", uselist=False)
    )

    __table_args__ = (
        db.UniqueConstraint("series_id", "character_name", name="series_character"),
    )


class Process(db.Model):

    __tablename__ = "processes"

    process_id = db.Column(db.Integer, primary_key=True)
    process_name = db.Column(db.String(32), nullable=False)
    airflow_dag_id = db.Column(db.String(32), nullable=False)
    timestamp_add_process = db.Column(db.String(10), nullable=False)
    triggers = db.relationship("Trigger", backref="Process", lazy="dynamic")


class Trigger(db.Model):

    __tablename__ = "triggers"

    trigger_id = db.Column(db.Integer, primary_key=True)
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"), nullable=False)
    process_id = db.Column(
        db.Integer, db.ForeignKey("processes.process_id"), nullable=False
    )
    timestamp_add_trigger = db.Column(db.String(10), nullable=False)
    status_complete = db.Column(db.Boolean, nullable=False)

    __table_args__ = (
        db.UniqueConstraint("video_id", "process_id", name="video_process"),
    )


class Session_Info(db.Model):

    __tablename__ = "session_info"

    session_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    ip_timestamp = db.Column(db.String(50), nullable=False)
    ip = db.Column(db.String(16), nullable=False)
    timestamp = db.Column(db.Integer, nullable=False)
    route_rule = db.Column(db.String(100), nullable=False)


class Haar_Cascade(db.Model):

    __tablename__ = "haar_cascades"

    haar_cascade_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    haar_cascade_name = db.Column(db.String(32), primary_key=True)
    file = db.Column(db.String(48), primary_key=True)


class Object_PCA_Features(db.Model):

    __tablename__ = "object_pca_features"

    object_pca_features_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    object_id = db.Column(db.Integer, db.ForeignKey("objects.object_id"))
    feature_1 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_2 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_3 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_4 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_5 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_6 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_7 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_8 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_9 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_10 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_11 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_12 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_13 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_14 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_15 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_16 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_17 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_18 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_19 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_20 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_21 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_22 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_23 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_24 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_25 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_26 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_27 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_28 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_29 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_30 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_31 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_32 = db.Column(db.Numeric(7, 6), nullable=False)
    timestamp_added = db.Column(db.String(10), nullable=False)


class Object_UMAP_Features(db.Model):

    __tablename__ = "object_umap_features"

    object_umap_features_id = db.Column(
        db.Integer, primary_key=True, autoincrement=True
    )
    object_id = db.Column(db.Integer, db.ForeignKey("objects.object_id"))
    feature_1 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_2 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_3 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_4 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_5 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_6 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_7 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_8 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_9 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_10 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_11 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_12 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_13 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_14 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_15 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_16 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_17 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_18 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_19 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_20 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_21 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_22 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_23 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_24 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_25 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_26 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_27 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_28 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_29 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_30 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_31 = db.Column(db.Numeric(7, 6), nullable=False)
    feature_32 = db.Column(db.Numeric(7, 6), nullable=False)
    timestamp_added = db.Column(db.String(10), nullable=False)


class Facenet_Haar_Model(db.Model):

    __tablename__ = "facenet_haar_models"

    facenet_haar_model_id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    model_name = db.Column(db.String(24), unique=True, nullable=False)
    series_id = db.Column(db.Integer, db.ForeignKey("series.series_id"))
    timestamp_added = db.Column(db.String(10), nullable=False)
    model_input_data = db.relationship(
        "Facenet_Haar_Model_Input_Data", backref="Facenet_Haar_Model", lazy="dynamic"
    )
    model_predictions = db.relationship(
        "Facenet_Haar_Prediction", backref="Facenet_Haar_Model", lazy="dynamic"
    )


class Facenet_Haar_Model_Input_Data(db.Model):

    __tablename__ = "facenet_haar_model_input_data"

    facenet_haar_model_input_data_id = db.Column(
        db.Integer, primary_key=True, autoincrement=True
    )
    object_id = db.Column(
        db.Integer, db.ForeignKey("objects.object_id"), nullable=False
    )
    character_id = db.Column(
        db.Integer, db.ForeignKey("characters.character_id"), nullable=False
    )
    model_name = db.Column(
        db.String(24), db.ForeignKey("facenet_haar_models.model_name"), nullable=False
    )
    timestamp_added = db.Column(db.String(10), nullable=False)

    __table_args__ = (
        db.UniqueConstraint(
            "model_name", "object_id", name="facenet_haar_model_input_model_object"
        ),
    )


class Facenet_Haar_Prediction(db.Model):

    __tablename__ = "facenet_haar_predictions"

    facenet_haar_prediction_id = db.Column(
        db.Integer, primary_key=True, autoincrement=True
    )
    object_id = db.Column(
        db.Integer, db.ForeignKey("objects.object_id"), nullable=False
    )
    object_complete_id = db.Column(
        db.Integer, db.ForeignKey("objects_complete.object_id"), nullable=True
    )
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"), nullable=False)
    character_id = db.Column(
        db.Integer, db.ForeignKey("characters.character_id"), nullable=False
    )
    facenet_haar_model_name = db.Column(
        db.String(24), db.ForeignKey("facenet_haar_models.model_name"), nullable=False
    )
    timestamp_added = db.Column(db.String(10), nullable=False)

    __table_args__ = (
        db.UniqueConstraint(
            "object_id",
            "video_id",
            "facenet_haar_model_name",
            name="facenet_haar_prediction_and_model",
        ),
        db.UniqueConstraint(
            "facenet_haar_model_name",
            "object_id",
            name="facenet_haar_prediction_model_object",
        ),
    )


class Facenet_Haar_Prediction_Complete(db.Model):

    __tablename__ = "facenet_haar_predictions_complete"

    facenet_haar_prediction_complete_id = db.Column(
        db.Integer, primary_key=True, autoincrement=True
    )
    object_id = db.Column(
        db.Integer, db.ForeignKey("objects_complete.object_id"), nullable=False
    )
    video_id = db.Column(db.Integer, db.ForeignKey("videos.video_id"), nullable=False)
    character_id = db.Column(
        db.Integer, db.ForeignKey("characters.character_id"), nullable=False
    )
    facenet_haar_model_name = db.Column(db.String(24), nullable=False)
    timestamp_added = db.Column(db.String(10), nullable=False)

    __table_args__ = (
        db.UniqueConstraint(
            "object_id", "video_id", name="facenet_haar_prediction_complete"
        ),
        db.UniqueConstraint(
            "facenet_haar_model_name",
            "object_id",
            name="facenet_haar_prediction_complete_model",
        ),
    )



"""added dlib face tables 2

Revision ID: 6b385de534b7
Revises: c56cae55295e
Create Date: 2020-06-03 13:54:49.975618

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import mysql

# revision identifiers, used by Alembic.
revision = "6b385de534b7"
down_revision = "c56cae55295e"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "dlib_faces_aligned",
        sa.Column("dlib_faces_aligned_id", sa.Integer(), nullable=False),
        sa.Column("video_id", sa.Integer(), nullable=True),
        sa.Column("frame_id", sa.Integer(), nullable=True),
        sa.Column("episode_id", sa.Integer(), nullable=True),
        sa.Column("object_number", sa.Integer(), nullable=False),
        sa.Column("object_x", sa.Integer(), nullable=False),
        sa.Column("object_y", sa.Integer(), nullable=False),
        sa.Column("object_w", sa.Integer(), nullable=False),
        sa.Column("object_h", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["episode_id"],
            ["episodes.episode_id"],
        ),
        sa.ForeignKeyConstraint(
            ["frame_id"],
            ["frames.frame_id"],
        ),
        sa.ForeignKeyConstraint(
            ["video_id"],
            ["videos.video_id"],
        ),
        sa.PrimaryKeyConstraint("dlib_faces_aligned_id"),
        sa.UniqueConstraint(
            "video_id", "frame_id", "object_number", name="dlib_faces_aligned"
        ),
    )
    op.create_table(
        "dlib_faces_unaligned",
        sa.Column("dlib_faces_unaligned_id", sa.Integer(), nullable=False),
        sa.Column("video_id", sa.Integer(), nullable=True),
        sa.Column("frame_id", sa.Integer(), nullable=True),
        sa.Column("episode_id", sa.Integer(), nullable=True),
        sa.Column("object_number", sa.Integer(), nullable=False),
        sa.Column("object_x", sa.Integer(), nullable=False),
        sa.Column("object_y", sa.Integer(), nullable=False),
        sa.Column("object_w", sa.Integer(), nullable=False),
        sa.Column("object_h", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["episode_id"],
            ["episodes.episode_id"],
        ),
        sa.ForeignKeyConstraint(
            ["frame_id"],
            ["frames.frame_id"],
        ),
        sa.ForeignKeyConstraint(
            ["video_id"],
            ["videos.video_id"],
        ),
        sa.PrimaryKeyConstraint("dlib_faces_unaligned_id"),
        sa.UniqueConstraint(
            "video_id", "frame_id", "object_number", name="dlib_face_unaligned"
        ),
    )
    op.drop_index("airflow_dag_id", table_name="processes")
    op.drop_index("process_name", table_name="processes")
    op.drop_column("processes", "process_order")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "processes",
        sa.Column(
            "process_order",
            mysql.INTEGER(display_width=11),
            autoincrement=False,
            nullable=True,
        ),
    )
    op.create_index("process_name", "processes", ["process_name"], unique=True)
    op.create_index("airflow_dag_id", "processes", ["airflow_dag_id"], unique=True)
    op.drop_table("dlib_faces_unaligned")
    op.drop_table("dlib_faces_aligned")
    # ### end Alembic commands ###
